//
// Module: 	CAM_generic_9s
// Design: 	CAM_Top
// Verilog code:	Hierarchical RTL
//			Instantiate CAM_RAMB16
//				Instantiate INIT_RAMB16_S1_S36
//				Instantiate INIT_9_RAM32x1s
//			Instantiate ENCODE_5_LSB
//			Instantiate DECODE_X
//			Instantiate ENCODE_X_MSB
//		Choose the right ENCODE and DECODE modules according to the number of CAM32x9s
//		If "nb_cam32x9s" = 2 then ENCODE_1_MSB and DECODE_1 must be used: 1 bit to decode 2 CAM32x9s
//		If "nb_cam32x9s" = 4 then ENCODE_2_MSB and DECODE_2 must be used: 2 bits to decode 4 CAM32x9s
//		If "nb_cam32x9s" = 8 then ENCODE_3_MSB and DECODE_3 must be used: 3 bits to decode 8 CAM32x9s
//		If "nb_cam32x9s" = 16 then ENCODE_4_MSB and DECODE_4 must be used: 4 bits to decode 16 CAM32x9s
//		Note: Configuration is not supported by synthesis tools
//
//
// Description: Instantiate "nb_cam32x9s" CAM_RAMB16 (see generic)
//		"nb_cam32x9s" x 32 words depth x 9 bits width
//		1 clock cycle Read (or Match), 
//		2 clock cycles Write (Erase on the first clock then Store on the second) / If only 1 clock cycle => Erase Only.
//		MATCH_OK indicates one or more matches is/are found.
//		MATCH_ADDR ouput the address of the match, if ONLY ONE is found
//		ADDR_VALID indicates when MATCH_ADDR is a valid address (Optional)
//
// Device: 	VIRTEX-II Family
//		modules CAM_RAMB16 fits in 1 BlockRAM column (+ CLB)
//		
// Synthesis_tool	Synopsys FPGA Express ver. 3.6 
//	 	        Enable Synthesis Option: Verilog Pre-procesor
//
//
//
// Created by: Lakshmi Gopalakrishnan & Jean-Louis BRELET / XILINX - VIRTEX-II Applications
// Date: July 25, 2001
// Version: 1.0
//
//
//   Disclaimer:  THESE DESIGNS ARE PROVIDED "AS IS" WITH NO WARRANTY 
//                WHATSOEVER AND XILINX SPECIFICALLY DISCLAIMS ANY 
//                IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS FOR
//                A PARTICULAR PURPOSE, OR AGAINST INFRINGEMENT.
//
//  Copyright (c) 1999 Xilinx, Inc.  All rights reserved.
//////////////////////////////////////////////////////////////////////////////////////////////////////-
`include "parameters.v"

`timescale 1 ns/100 ps

module CAM_generic_9s (DATA_IN,       // Data to compare or to write
                       ADDR,          // Address only when write
                       WRITE_ENABLE,  // Write Enable if high (2 clock cycles)
                       CLK, 
                       MATCH_ENABLE,  //Enable to find a match,else No change on Match Bus
                       MATCH_RST,     //Synchronous.If'0'MATCH bus is zero,when MATCH_ENABLE=1
                       GLOBAL_RST,    //Global Asynchronous FF Reset
                       R_MATCH_ADDR,  //Registered Match address found
            //`ifdef use_addr_valid
            //           ADDR_VALID,     //Double the size of the encoder.Need only if simultaneous matches can occur
            //`endif
                       R_MATCH_OK     // '1' if MATCH found. Registered
                       );



input CLK, WRITE_ENABLE, MATCH_ENABLE, MATCH_RST, GLOBAL_RST;
input [8:0] DATA_IN;
input [`addr_width-1:0] ADDR;
output [`addr_width-1:0] R_MATCH_ADDR;
output R_MATCH_OK;
//`ifdef use_addr_valid
 //         output ADDR_VALID;
//`endif

reg [`addr_width-1:0] R_MATCH_ADDR;
reg R_MATCH_OK;
//`ifdef use_addr_valid
//          reg ADDR_VALID;
//`endif

// Internal Signal Declarations
wire ERASE_WRITE;                         // Erase then Write Enable (2 clock cycles) 
reg WRITE_RAM;                           // Write enable in the RAM32x1s
wire [`nb_cam32x9s-1:0] BUS_WRITE_ENABLE; // Decode the CAM32x1s to be addressed
//`ifdef use_addr_valid
//     wire [`addr_width-1:0] BUS_ADDR_VALID; // Optional
//`endif
wire [`nb_cam32x9s-1:0] BUS_MATCH_OK;
wire [`addr_width-1:0] MATCH_ADDR;        // Match address found
wire MATCH_OK;                           // '1' if MATCH found
wire [31:0] MATCH_ARRAY_0, MATCH_ARRAY_1;
//If nb_cam32x9s = 16 then 16 MATCH_ARRAY and ADDR_ARRAY signals need to be declared.
//wire [31:0] MATCH_ARRAY_0, MATCH_ARRAY_1,MATCH_ARRAY_2,MATCH_ARRAY_3,MATCH_ARRAY_4,MATCH_ARRAY_5,MATCH_ARRAY_6,MATCH_ARRAY_7,
//            MATCH_ARRAY_8,MATCH_ARRAY_9,MATCH_ARRAY_10,MATCH_ARRAY_11,MATCH_ARRAY_12,MATCH_ARRAY_13,MATCH_ARRAY_14,MATCH_ARRAY_15;
//wire [4:0] ADDR_ARRAY_0, ADDR_ARRAY_1,ADDR_ARRAY_2,ADDR_ARRAY_3,ADDR_ARRAY_4,ADDR_ARRAY_5,ADDR_ARRAY_6,ADDR_ARRAY_7,
//            ADDR_ARRAY_8,ADDR_ARRAY_9,ADDR_ARRAY_10,ADDR_ARRAY_11,ADDR_ARRAY_12,ADDR_ARRAY_13,ADDR_ARRAY_14,ADDR_ARRAY_15; 
wire [4:0] ADDR_ARRAY_0, ADDR_ARRAY_1;


// In the "Write Enable" mode, the first clock cycle erases the data, then in the 2nd clock cycle the new data is written.
// A 1 clock cycle WRITE_ENABLE erases only the data.
// RAM32x1s are read to erase the RAMB16 ONLY, used to find a match.
// Does not care if the data is not erased in the RAM32x1s
// Generate a Write Enable for the RAM32x1s


// CYCLE_ERASE
always @ (posedge CLK or posedge GLOBAL_RST)

  begin
        if (GLOBAL_RST)
          WRITE_RAM = 1'b0;
        else if (WRITE_ENABLE == 1'b0)
          WRITE_RAM = 1'b0;
        else
          WRITE_RAM = !(WRITE_RAM);
          
  end  // CYCLE_ERASE
  
// Generate an Erase signal '0' then a Write signal '1' for the RAMB16
// If WRITE_ENABLE is deasserted after 1 clock cycle, the WRITE is not valid (WRITE_ENABLE is used for WEA port)

assign ERASE_WRITE = !(WRITE_RAM);

//Create the Write Enable signal for each CAM_RAMB16 = CAM 64 words

DECODE_1 DECODE_X (.ADDR(ADDR[`addr_width-1:5]), .ENABLE(WRITE_ENABLE), .BINARY_ADDR(BUS_WRITE_ENABLE));

//Create the write enable signal for each CAM_RAMB16 = CAM 128 words
//DECODE_2 DECODE_X (.ADDR(ADDR[`addr_width-1:5]), .ENABLE(WRITE_ENABLE), .BINARY_ADDR(BUS_WRITE_ENABLE));

//Create the write enable signal for each CAM_RAMB16 = CAM 256 words
//DECODE_3 DECODE_X (.ADDR(ADDR[`addr_width-1:5]), .ENABLE(WRITE_ENABLE), .BINARY_ADDR(BUS_WRITE_ENABLE));

//Create the write enable signal for each CAM_RAMB16 = CAM 512 words
//DECODE_4 DECODE_X (.ADDR(ADDR[`addr_width-1:5]), .ENABLE(WRITE_ENABLE), .BINARY_ADDR(BUS_WRITE_ENABLE));

//CAM_RAMB16 and ENCODE_X_LSB instantiation. These have to be instantiated as many times as nb_cam32x9s. 
//In this case it is 2, therefore just 2 instantiations.

CAM_RAMB16 CAM_X_RAMB16_0 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), .WRITE_ENABLE(BUS_WRITE_ENABLE[0]),
                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_0));
// Generate the Match Address                         
ENCODE_5_LSB ENCODE_X_LSB_0 (.BINARY_ADDR(MATCH_ARRAY_0), 
                             .MATCH_ADDR(ADDR_ARRAY_0),
                  //`ifdef use_addr_valid 
                  //           .ADDR_VALID(BUS_ADDR_VALID[0]),
                  //`endif
                             .MATCH_OK(BUS_MATCH_OK[0]));
                             
CAM_RAMB16 CAM_X_RAMB16_1 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), .WRITE_ENABLE(BUS_WRITE_ENABLE[1]),
                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_1));
                         
// Generate the Match Address                         
ENCODE_5_LSB ENCODE_X_LSB_1 (.BINARY_ADDR(MATCH_ARRAY_1), 
                             .MATCH_ADDR(ADDR_ARRAY_1),
                  //`ifdef use_addr_valid 
                  //           .ADDR_VALID(BUS_ADDR_VALID[1]),
                  //`endif 
                             .MATCH_OK(BUS_MATCH_OK[1]));


//If the value of nb_cam32x9s = 16, then 16 instantiations required.                             
//CAM_RAMB16 CAM_X_RAMB16_2 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[2]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_2));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_2 (.BINARY_ADDR(MATCH_ARRAY_2), .MATCH_ADDR(ADDR_ARRAY_2), 
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[2]),
//                  `endif  
//                             .MATCH_OK(BUS_MATCH_OK[2]));                             

//CAM_RAMB16 CAM_X_RAMB16_3 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[3]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_3));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_3 (.BINARY_ADDR(MATCH_ARRAY_3), .MATCH_ADDR(ADDR_ARRAY_3),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[3]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[3]));

//CAM_RAMB16 CAM_X_RAMB16_4 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[4]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_4));
 
// Generate the Match Address                        
//ENCODE_5_LSB ENCODE_X_LSB_4 (.BINARY_ADDR(MATCH_ARRAY_4), .MATCH_ADDR(ADDR_ARRAY_4),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[4]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[4]));

//CAM_RAMB16 CAM_X_RAMB16_5 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[5]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_5));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_5 (.BINARY_ADDR(MATCH_ARRAY_5), .MATCH_ADDR(ADDR_ARRAY_5), 
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[5]),
//                  `endif  
//                             .MATCH_OK(BUS_MATCH_OK[5]));

//CAM_RAMB16 CAM_X_RAMB16_6 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[6]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_6));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_6 (.BINARY_ADDR(MATCH_ARRAY_6), .MATCH_ADDR(ADDR_ARRAY_6),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[6]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[6]));

//CAM_RAMB16 CAM_X_RAMB16_7 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[7]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_7));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_7 (.BINARY_ADDR(MATCH_ARRAY_7), .MATCH_ADDR(ADDR_ARRAY_7),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[7]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[7]));

//CAM_RAMB16 CAM_X_RAMB16_8 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[8]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_8));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_8 (.BINARY_ADDR(MATCH_ARRAY_8), .MATCH_ADDR(ADDR_ARRAY_8),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[8]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[8]));

//CAM_RAMB16 CAM_X_RAMB16_9 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[9]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_9));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_9 (.BINARY_ADDR(MATCH_ARRAY_9), .MATCH_ADDR(ADDR_ARRAY_9),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[9]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[9]));

//CAM_RAMB16 CAM_X_RAMB16_10 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[10]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_10));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_10 (.BINARY_ADDR(MATCH_ARRAY_10), .MATCH_ADDR(ADDR_ARRAY_10),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[10]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[10]));

//CAM_RAMB16 CAM_X_RAMB16_11 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[11]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_11));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_11 (.BINARY_ADDR(MATCH_ARRAY_11), .MATCH_ADDR(ADDR_ARRAY_11),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[11]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[11]));

//CAM_RAMB16 CAM_X_RAMB16_12 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[12]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_12));

// Generate the Match Address                        
//ENCODE_5_LSB ENCODE_X_LSB_12 (.BINARY_ADDR(MATCH_ARRAY_12), .MATCH_ADDR(ADDR_ARRAY_12),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[12]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[12]));

//CAM_RAMB16 CAM_X_RAMB16_13 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[13]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_13));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_13 (.BINARY_ADDR(MATCH_ARRAY_13), .MATCH_ADDR(ADDR_ARRAY_13),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[13]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[13]));

//CAM_RAMB16 CAM_X_RAMB16_14 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[14]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_14));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_14 (.BINARY_ADDR(MATCH_ARRAY_14), .MATCH_ADDR(ADDR_ARRAY_14),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[14]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[14]));

//CAM_RAMB16 CAM_X_RAMB16_15 (.DATA_IN(DATA_IN[8:0]), .ADDR(ADDR[4:0]), WRITE_ENABLE(BUS_WRITE_ENABLE[15]),
//                         .ERASE_WRITE(ERASE_WRITE), .WRITE_RAM(WRITE_RAM), .CLK(CLK), .MATCH_ENABLE(MATCH_ENABLE),
//                         .MATCH_RST(MATCH_RST), .MATCH(MATCH_ARRAY_15));

// Generate the Match Address                         
//ENCODE_5_LSB ENCODE_X_LSB_15 (.BINARY_ADDR(MATCH_ARRAY_15), .MATCH_ADDR(ADDR_ARRAY_15),
//                  `ifdef use_addr_valid 
//                             .ADDR_VALID(BUS_ADDR_VALID[15]),
//                  `endif   
//                             .MATCH_OK(BUS_MATCH_OK[15]));


//Generate Top address CAM 64 words

ENCODE_1_MSB ENCODE_X_MSB (.BINARY_ADDR(BUS_MATCH_OK), 
                             .ADDR_LSB_0(ADDR_ARRAY_0), 
                             .ADDR_LSB_1(ADDR_ARRAY_1),
                             .MATCH_ADDR(MATCH_ADDR[`addr_width-1:0]),
                   //`ifdef use_addr_valid 
                   //          .ADDR_VALID(ADDR_VALID),
                   //`endif
                             .MATCH_OK(MATCH_OK));
                             
//Generate Top address CAM 128 words
//ENCODE_2_MSB ENCODE_X_MSB (.BINARY_ADDR(BUS_MATCH_OK), 
//                             .ADDR_LSB_0(ADDR_ARRAY_0), 
//                             .ADDR_LSB_1(ADDR_ARRAY_1),
//                             .ADDR_LSB_2(ADDR_ARRAY_2),
//                             .ADDR_LSB_3(ADDR_ARRAY_3),
//                             .MATCH_ADDR(MATCH_ADDR[`addr_width-1:0]), 
//                   `ifdef use_addr_valid 
//                             .ADDR_VALID(ADDR_VALID),
//                   `endif
//                             .MATCH_OK(MATCH_OK));

//Generate Top address CAM 256 words
//ENCODE_3_MSB ENCODE_X_MSB (.BINARY_ADDR(BUS_MATCH_OK), 
//                             .ADDR_LSB_0(ADDR_ARRAY_0), 
//                             .ADDR_LSB_1(ADDR_ARRAY_1),
//                             .ADDR_LSB_2(ADDR_ARRAY_2),
//                             .ADDR_LSB_3(ADDR_ARRAY_3),
//                             .ADDR_LSB_4(ADDR_ARRAY_4),
//                             .ADDR_LSB_5(ADDR_ARRAY_5),
//                             .ADDR_LSB_6(ADDR_ARRAY_6),
//                             .ADDR_LSB_7(ADDR_ARRAY_7),
//                             .MATCH_ADDR(MATCH_ADDR[`addr_width-1:0]),
//                   `ifdef use_addr_valid 
//                             .ADDR_VALID(ADDR_VALID),
//                   `endif 
//                             .MATCH_OK(MATCH_OK));

//Generate Top address CAM 512 words
//ENCODE_4_MSB ENCODE_X_MSB (.BINARY_ADDR(BUS_MATCH_OK), 
//                             .ADDR_LSB_0(ADDR_ARRAY_0), 
//                             .ADDR_LSB_1(ADDR_ARRAY_1),
//                             .ADDR_LSB_2(ADDR_ARRAY_2),
//                             .ADDR_LSB_3(ADDR_ARRAY_3),
//                             .ADDR_LSB_4(ADDR_ARRAY_4),
//                             .ADDR_LSB_5(ADDR_ARRAY_5),
//                             .ADDR_LSB_6(ADDR_ARRAY_6),
//                             .ADDR_LSB_7(ADDR_ARRAY_7),
//                             .ADDR_LSB_8(ADDR_ARRAY_8),
//                             .ADDR_LSB_9(ADDR_ARRAY_9),
//                             .ADDR_LSB_10(ADDR_ARRAY_10),
//                             .ADDR_LSB_11(ADDR_ARRAY_11),
//                             .ADDR_LSB_12(ADDR_ARRAY_12),
//                             .ADDR_LSB_13(ADDR_ARRAY_13),
//                             .ADDR_LSB_14(ADDR_ARRAY_14),
//                             .ADDR_LSB_15(ADDR_ARRAY_15),
//                             .MATCH_ADDR(MATCH_ADDR[`addr_width-1:0]),
//                   `ifdef use_addr_valid 
//                             .ADDR_VALID(ADDR_VALID),
//                   `endif 
//                             .MATCH_OK(MATCH_OK));



//Registered Outputs

always @ (posedge CLK or posedge GLOBAL_RST)
  begin
     if (GLOBAL_RST)
      begin
        R_MATCH_ADDR = 0;
        R_MATCH_OK = 0;
      end
     else
      begin
        R_MATCH_ADDR = MATCH_ADDR;
        R_MATCH_OK = MATCH_OK;
      end
  end     //Registered Outputs
  
endmodule //CAM_generic_9s   



  
  






